// minecraft-module-proxy:@minecraft/server
import { EquipmentSlot } from "@minecraft/server";
import { system } from "@minecraft/server";
import { world } from "@minecraft/server";

// src/runtime/configs/items.ts
var config = {
  jetpack: {
    vertical_speed: 0.23,
    horizontal_push: 0.23,
    sprint_multiplier_horizontal: 1,
    slowfall_amount: 3,
    bar_char_code: "\u258C",
    bar_empty_code: "\xA7c",
    bar_fuel_code: "\xA7a",
    max_charges: 8,
    consume_time: 2,
    recharge_time: 0.5
  }
};

// src/runtime/utils/vec3.ts
var Vec3 = class _Vec3 {
  constructor(x, y, z) {
    this.x = x;
    this.y = y;
    this.z = z;
  }
  static withX(v, x) {
    return new _Vec3(x, v.y, v.z);
  }
  withX(x) {
    return new _Vec3(x, this.y, this.z);
  }
  static withY(v, y) {
    return new _Vec3(v.x, y, v.z);
  }
  withY(y) {
    return new _Vec3(this.x, y, this.z);
  }
  static withZ(v, z) {
    return new _Vec3(v.x, v.y, z);
  }
  withZ(z) {
    return new _Vec3(this.x, this.y, z);
  }
  min(v) {
    return new _Vec3(this.x < v.x ? this.x : v.x, this.y < v.y ? this.y : v.y, this.z < v.z ? this.z : v.z);
  }
  static min(v1, v2) {
    return new _Vec3(v1.x < v2.x ? v1.x : v2.x, v1.y < v2.y ? v1.y : v2.y, v1.z < v2.z ? v1.z : v2.z);
  }
  max(v) {
    return new _Vec3(this.x > v.x ? this.x : v.x, this.y > v.y ? this.y : v.y, this.z > v.z ? this.z : v.z);
  }
  static max(v1, v2) {
    return new _Vec3(v1.x > v2.x ? v1.x : v2.x, v1.y > v2.y ? v1.y : v2.y, v1.z > v2.z ? v1.z : v2.z);
  }
  static abs(v) {
    return new _Vec3(Math.abs(v.x), Math.abs(v.y), Math.abs(v.z));
  }
  abs() {
    return new _Vec3(Math.abs(this.x), Math.abs(this.y), Math.abs(this.z));
  }
  static add(v1, v2) {
    return new _Vec3(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
  }
  add(v) {
    return new _Vec3(this.x + v.x, this.y + v.y, this.z + v.z);
  }
  static subtract(v1, v2) {
    return new _Vec3(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
  }
  subtract(v) {
    return new _Vec3(this.x - v.x, this.y - v.y, this.z - v.z);
  }
  static multiply(v1, v2) {
    return new _Vec3(v1.x * v2.x, v1.y * v2.y, v1.z * v2.z);
  }
  multiply(v) {
    return new _Vec3(this.x * v.x, this.y * v.y, this.z * v.z);
  }
  static equals(v1, v2) {
    return v1.x === v2.x && v1.y === v2.y && v1.z === v2.z;
  }
  equals(v) {
    return this.x === v.x && this.y === v.y && this.z === v.z;
  }
  static scale(v, scalar) {
    return new _Vec3(v.x * scalar, v.y * scalar, v.z * scalar);
  }
  scale(scalar) {
    return new _Vec3(this.x * scalar, this.y * scalar, this.z * scalar);
  }
  static normal(v) {
    const length = Math.sqrt(v.x * v.x + v.y * v.y + v.z * v.z);
    return new _Vec3(v.x / length, v.y / length, v.z / length);
  }
  normal() {
    const length = this.mag();
    return new _Vec3(this.x / length, this.y / length, this.z / length);
  }
  static mag(v) {
    return Math.sqrt(v.x * v.x + v.y * v.y + v.z * v.z);
  }
  mag() {
    return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
  }
  static distance(v1, v2) {
    return _Vec3.subtract(v1, v2).mag();
  }
  distance(v) {
    return this.subtract(v).mag();
  }
  static dot(v1, v2) {
    return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
  }
  dot(v) {
    return this.x * v.x + this.y * v.y + this.z * v.z;
  }
  static cross(v1, v2) {
    return new _Vec3(v1.y * v2.z - v1.z * v2.y, v1.z * v2.x - v1.x * v2.z, v1.x * v2.y - v1.y * v2.x);
  }
  cross(v) {
    return new _Vec3(this.y * v.z - this.z * v.y, this.z * v.x - this.x * v.z, this.x * v.y - this.y * v.x);
  }
  static lerp(v1, v2, alpha) {
    return v3.wrap(v1).lerp(v2, alpha);
  }
  lerp(v, alpha) {
    return this.scale(1 - alpha).add(v3.wrap(v).scale(alpha));
  }
  static floor(v) {
    return new _Vec3(Math.floor(v.x), Math.floor(v.y), Math.floor(v.z));
  }
  static rotateY(v, angle) {
    const x = v.x * Math.cos(angle) - v.z * Math.sin(angle);
    const z = v.x * Math.sin(angle) + v.z * Math.cos(angle);
    return new _Vec3(x, v.y, z);
  }
  rotateY(angle) {
    const x = this.x * Math.cos(angle) - this.z * Math.sin(angle);
    const z = this.x * Math.sin(angle) + this.z * Math.cos(angle);
    return new _Vec3(x, this.y, z);
  }
  round() {
    return new _Vec3(Math.round(this.x), Math.round(this.y), Math.round(this.z));
  }
  static round(v) {
    return new _Vec3(Math.round(v.x), Math.round(v.y), Math.round(v.z));
  }
  floor() {
    return new _Vec3(Math.floor(this.x), Math.floor(this.y), Math.floor(this.z));
  }
  clone() {
    return new _Vec3(this.x, this.y, this.z);
  }
  toString() {
    return `Vector3<${this.x}, ${this.y}, ${this.z}>`;
  }
  unwrap() {
    return { x: this.x, y: this.y, z: this.z };
  }
  length() {
    return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
  }
  static Up() {
    return new _Vec3(0, 1, 0);
  }
  static Down() {
    return new _Vec3(0, -1, 0);
  }
  static North() {
    return new _Vec3(0, 0, -1);
  }
  static South() {
    return new _Vec3(0, 0, 1);
  }
  static East() {
    return new _Vec3(1, 0, 0);
  }
  static West() {
    return new _Vec3(-1, 0, 0);
  }
  static Zero() {
    return new _Vec3(0, 0, 0);
  }
  static One() {
    return new _Vec3(1, 1, 1);
  }
};
var v3 = (x, y, z) => {
  return new Vec3(x, y, z);
};
v3.wrap = (v) => new Vec3(v.x, v.y, v.z);
v3.up = v3(0, 1, 0);
v3.down = v3(0, -1, 0);
v3.north = v3(0, 0, -1);
v3.south = v3(0, 0, 1);
v3.east = v3(1, 0, 0);
v3.west = v3(-1, 0, 0);
v3.zero = v3(0, 0, 0);
v3.one = v3(1, 1, 1);

// src/runtime/utils/util.ts
var clamp = function(value, min, max) {
  return Math.max(Math.min(value, max), min);
};
var Dimensions = [
  world.getDimension("minecraft:overworld"),
  world.getDimension("minecraft:nether"),
  world.getDimension("minecraft:the_end")
];
function playAnimationFactory(animation, opts) {
  let commandArgs = [
    animation,
    opts.nextState,
    opts.blendOutTime,
    opts.stopExpression ? `"${opts.stopExpression}"` : void 0,
    opts.controller
  ];
  let skipped = false;
  let lastIdx = 1;
  for (let i = 1; i < commandArgs.length; i++) {
    if (commandArgs[i] === void 0) {
      skipped = true;
    } else {
      if (skipped) {
        throw "playanimation encounterd data after a missing optional parameter";
      }
      lastIdx = i;
    }
  }
  let command = `playanimation @s ${commandArgs.join(" ")}`;
  return command;
}

// src/runtime/systems/items/jetpack.ts
var equippedPlayers = /* @__PURE__ */ new Set();
var jetpackFuel = /* @__PURE__ */ new Map();
var soundLoop = /* @__PURE__ */ new Map();
world.afterEvents.playerSpawn.subscribe(function processPlayerSpawn(arg) {
  let player = arg.player;
  if (!hasJetpackEquipped(player))
    return;
  registerPlayer(player);
});
world.afterEvents.playerLeave.subscribe(function processPlayerLeave(arg) {
  removePlayer(arg.playerId);
});
function registerPlayer(player) {
  if (equippedPlayers.has(player.id))
    return;
  equippedPlayers.add(player.id);
  jetpackFuel.set(player.id, {
    fuel: config.jetpack.max_charges,
    ground: config.jetpack.recharge_time * 20,
    air: config.jetpack.consume_time * 20
  });
  player.runCommandAsync(
    playAnimationFactory("animation.spark_amm3.empty_animation", {
      nextState: "animation.spark_amm3.jetpack.third.take_off",
      blendOutTime: 0.4,
      controller: `controller.animation.spark_amm3.player_jetpack_third`,
      stopExpression: "!q.is_on_ground && q.is_item_name_any('slot.armor.chest', 'spark_amm3:jetpack')"
    })
  );
  player.runCommandAsync(
    playAnimationFactory("animation.spark_amm3.jetpack.third.landing", {
      nextState: "animation.spark_amm3.empty_animation",
      blendOutTime: 0.4,
      controller: `controller.animation.spark_amm3.player_jetpack_third`,
      stopExpression: "!q.is_item_name_any('slot.armor.chest', 'spark_amm3:jetpack')"
    })
  );
  player.runCommandAsync(
    playAnimationFactory("animation.spark_amm3.jetpack.third.landing", {
      nextState: "animation.spark_amm3.empty_animation",
      blendOutTime: 0.4,
      controller: `controller.animation.spark_amm3.player_jetpack_third`,
      stopExpression: "q.any_animation_finished && q.is_item_name_any('slot.armor.chest', 'spark_amm3:jetpack')"
    })
  );
  player.runCommandAsync(
    playAnimationFactory("animation.spark_amm3.jetpack.third.in_flight", {
      nextState: "animation.spark_amm3.empty_animation",
      blendOutTime: 0.4,
      controller: `controller.animation.spark_amm3.player_jetpack_third`,
      stopExpression: "!q.is_item_name_any('slot.armor.chest', 'spark_amm3:jetpack')"
    })
  );
  player.runCommandAsync(
    playAnimationFactory("animation.spark_amm3.jetpack.third.in_flight", {
      nextState: "animation.spark_amm3.jetpack.third.landing",
      blendOutTime: 0.4,
      controller: `controller.animation.spark_amm3.player_jetpack_third`,
      stopExpression: "q.is_on_ground && q.is_item_name_any('slot.armor.chest', 'spark_amm3:jetpack')"
    })
  );
  player.runCommandAsync(
    playAnimationFactory("animation.spark_amm3.jetpack.third.take_off", {
      nextState: "animation.spark_amm3.empty_animation",
      blendOutTime: 0.4,
      controller: `controller.animation.spark_amm3.player_jetpack_third`,
      stopExpression: "!q.is_item_name_any('slot.armor.chest', 'spark_amm3:jetpack')"
    })
  );
  player.runCommandAsync(
    playAnimationFactory("animation.spark_amm3.jetpack.third.take_off", {
      nextState: "animation.spark_amm3.jetpack.third.in_flight",
      blendOutTime: 0.4,
      controller: `controller.animation.spark_amm3.player_jetpack_third`,
      stopExpression: "q.any_animation_finished && q.is_item_name_any('slot.armor.chest', 'spark_amm3:jetpack')"
    })
  );
}
function removePlayer(id) {
  if (equippedPlayers.has(id)) {
    equippedPlayers.delete(id);
  }
  if (jetpackFuel.has(id)) {
    jetpackFuel.delete(id);
  }
}
function tick() {
  world.getPlayers().forEach((p) => {
    if (equippedPlayers.has(p.id))
      return;
    if (hasJetpackEquipped(p)) {
      registerPlayer(p);
    }
  });
  equippedPlayers.forEach((v) => {
    let ent = world.getEntity(v);
    if (!ent)
      return;
    if (ent.typeId != "minecraft:player")
      return;
    processJetpackTick(ent);
  });
}
function processJetpackTick(player) {
  if (!equippedPlayers.has(player.id)) {
    return;
  }
  const jetpackEquipped = hasJetpackEquipped(player);
  let fuelLevels = jetpackFuel.get(player.id);
  if (fuelLevels.fuel <= 0 && !jetpackEquipped) {
    return;
  }
  if (!player.isOnGround) {
    fuelLevels.air -= 1;
    fuelLevels.ground = config.jetpack.recharge_time;
    if (fuelLevels.air < 0) {
      fuelLevels.fuel -= 1;
      fuelLevels.air = config.jetpack.consume_time * 20;
    }
  }
  if (player.isOnGround || !jetpackEquipped) {
    fuelLevels.air = config.jetpack.consume_time;
    fuelLevels.ground -= 1;
    if (fuelLevels.ground < 0) {
      fuelLevels.fuel += 1;
      fuelLevels.ground = config.jetpack.recharge_time * 20;
    }
  }
  fuelLevels.fuel = clamp(fuelLevels.fuel, 0, config.jetpack.max_charges);
  jetpackFuel.set(player.id, fuelLevels);
  if (!jetpackEquipped) {
    if (fuelLevels.fuel == config.jetpack.max_charges) {
      return;
    }
    drawBar(player, fuelLevels);
    return;
  }
  drawBar(player, fuelLevels);
  let hasSlowfall = player.getEffect("slow_falling");
  if (hasSlowfall) {
    if (player.isOnGround) {
      player.removeEffect("slow_falling");
    }
  }
  player.addEffect("slow_falling", 3, {
    amplifier: config.jetpack.slowfall_amount,
    showParticles: false
  });
  if (player.isOnGround) {
    if (!soundLoop.has(player.id)) {
      soundLoop.set(player.id, {
        loop: 0,
        overheat: false,
        warning: false
      });
    }
    let s = soundLoop.get(player.id);
    s.loop = 0;
    s.overheat = false;
    if (fuelLevels.fuel > 3) {
      s.warning = false;
    }
    soundLoop.set(player.id, s);
  }
  try {
    if (!player.isOnGround) {
      let view = player.getViewDirection();
      let pOffsetRight = { x: -0.35, y: 0.5, z: -0.1 };
      let rightRot = Vec3.rotateY(
        pOffsetRight,
        -Math.atan2(view.x, view.z)
      );
      let right = Vec3.add(player.location, rightRot);
      player.dimension.spawnParticle("spark_amm3:jetpack_fire", right);
      player.dimension.spawnParticle("spark_amm3:jetpack_trail", right);
      let pOffsetLeft = { x: 0.35, y: 0.5, z: -0.1 };
      let leftRot = Vec3.rotateY(
        pOffsetLeft,
        -Math.atan2(view.x, view.z)
      );
      let left = Vec3.add(player.location, leftRot);
      player.dimension.spawnParticle("spark_amm3:jetpack_fire", left);
      player.dimension.spawnParticle("spark_amm3:jetpack_trail", left);
      if (fuelLevels.fuel == 0) {
        let pOffsetCenter = { x: 0.35, y: 1.8, z: -0.1 };
        let centerRot = Vec3.rotateY(
          pOffsetCenter,
          -Math.atan2(view.x, view.z)
        );
        let center = Vec3.add(player.location, centerRot);
        player.dimension.spawnParticle(
          "spark_amm3:jetpack_smoke",
          center
        );
      }
      let soundTime = soundLoop.get(player.id);
      if (soundTime == void 0) {
        soundTime = {
          loop: 0,
          overheat: false,
          warning: false
        };
      }
      if (soundTime.loop == 0) {
        player.playSound("sound.spark_amm3.jetpack_startup");
      }
      if (soundTime.loop % 15 == 0) {
        player.playSound("sound.spark_amm3.jetpack_loop");
      }
      soundTime.loop++;
      if (fuelLevels.fuel == 1 && soundTime.warning == false) {
        player.playSound("sound.spark_amm3.jetpack_warning");
        soundTime.warning = true;
      }
      if (fuelLevels.fuel == 0 && soundTime.overheat == false) {
        player.playSound("sound.spark_amm3.jetpack_overheat");
        soundTime.overheat = true;
      }
      soundLoop.set(player.id, soundTime);
    }
  } catch {
    return;
  }
  if (fuelLevels.fuel == 0)
    return;
  if (player.isJumping) {
    let dir = player.getViewDirection();
    let norma = Vec3.normal(dir);
    let runMultiplier = player.isSprinting ? config.jetpack.sprint_multiplier_horizontal : 1;
    player.applyKnockback(
      norma.x,
      norma.z,
      config.jetpack.horizontal_push * runMultiplier,
      config.jetpack.vertical_speed
    );
  }
  if (player.isSneaking && !player.isOnGround) {
    let dir = player.getViewDirection();
    let norma = Vec3.normal(dir);
    let runMultiplier = player.isSprinting ? config.jetpack.sprint_multiplier_horizontal : 1;
    player.applyKnockback(
      norma.x,
      norma.z,
      config.jetpack.horizontal_push * runMultiplier,
      -config.jetpack.vertical_speed
    );
  }
  if (!player.isSneaking && !player.isJumping && !player.isOnGround) {
    let dir = player.getViewDirection();
    let norma = Vec3.normal(dir);
    let runMultiplier = player.isSprinting ? config.jetpack.sprint_multiplier_horizontal : 1;
    player.applyKnockback(
      norma.x,
      norma.z,
      config.jetpack.horizontal_push * runMultiplier,
      -0.05
    );
  }
}
function drawBar(player, fuel) {
  if (Math.floor(Date.now() / 50) % 5 != 0)
    return;
  let totalBars = config.jetpack.max_charges;
  let bar = [config.jetpack.bar_fuel_code];
  for (let i = 0; i < totalBars; i++) {
    if (i == fuel.fuel) {
      bar.push(config.jetpack.bar_empty_code);
    }
    bar.push(config.jetpack.bar_char_code);
  }
  let outputString = "";
  bar.forEach((v) => {
    outputString += v;
  });
  let raw = {
    text: outputString
  };
  player.onScreenDisplay.setActionBar(raw);
}
function hasJetpackEquipped(player) {
  let equipment = player.getComponent(
    "equippable"
  );
  if (!equipment)
    return false;
  let chestItem = equipment.getEquipment(EquipmentSlot.Chest);
  if (!chestItem)
    return false;
  if (chestItem.typeId != `${"spark_amm3"}:jetpack`)
    return false;
  return true;
}
system.runInterval(tick, 1);
var startJetpacks = 1;
var playerJoinTag = "spark_amm3.joined";
world.afterEvents.playerSpawn.subscribe((event) => {
  if (event.initialSpawn) {
    if (event.player.hasTag(playerJoinTag))
      return;
    event.player.addTag(playerJoinTag);
    event.player.runCommandAsync("gamerule sendcommandfeedback false");
    event.player.runCommandAsync(`give @s ${"spark_amm3"}:jetpack ${startJetpacks}`);
    event.player.runCommandAsync("gamerule sendcommandfeedback true");
    system.runTimeout(() => {
      event.player.sendMessage({
        rawtext: [
          {
            translate: "spark.amm3.chat.is_active"
          }
        ]
      });
    }, 100);
  }
});

// src/runtime/systems/items/main.ts
world.afterEvents.itemUse.subscribe(function processItemUse(arg) {
  let id = arg.itemStack.typeId;
  switch (id) {
    case `${"spark_amm3"}:jetpack`:
      processJetpack(arg);
  }
});
function processJetpack(arg) {
  registerPlayer(arg.source);
}
